# Phylogenetics

In the **Diversity** module'a **Phylogenetics** code, we generate Phylogenetic
diversity measures, based on Faith PD and extended by Chao.

#### Usage

Using the functionality in the package is simple:

 - Create a tree (using our Phylo package)
 - Create a PhyloTypes (AbstractTypes subtype) object from it
 - Create a Metacommunity from that
 - Calculate diversity!

```jldoctest
julia> using Phylo, Diversity
Creating Diversity to Phylo interface...

julia> species = ["Dog", "Cat", "Human", "Potato"];

julia> community = [4, 1, 3, 2] / 10;

julia> nt = rand(Nonultrametric(species))
LinkTree{OneRoot,String,LinkNode{OneRoot,String,Dict{String,Any},LinkBranch{OneRoot,String,Dict{String,Any},Float64}},LinkBranch{OneRoot,String,Dict{String,Any},Float64},Dict{String,Any}} with 4 tips, 7 nodes and 6 branches.
Leaf names are Dog, Cat, Potato and Human


julia> collect(getbranches(nt))
6-element Array{LinkBranch{OneRoot,String,Dict{String,Any},Float64},1}:
 LinkBranch 7, from node Node 5 to node Cat (length 0.9239010134651334).

 LinkBranch 8, from node Node 5 to node Potato (length 0.22314202428976326).

 LinkBranch 9, from node Node 6 to node Node 5 (length 0.8631183174331696).

 LinkBranch 10, from node Node 6 to node Human (length 0.00018168653247226503).

 LinkBranch 11, from node Node 7 to node Dog (length 0.20720995300152265).

 LinkBranch 12, from node Node 7 to node Node 6 (length 0.3064428959482562).


julia> ph = PhyloTypes(nt);

julia> metaphylo = Metacommunity(community, ph);

julia> leafnames = gettypenames(metaphylo, true)
4-element Array{String,1}:
 "Dog"
 "Cat"
 "Human"
 "Potato"

julia> meta_gamma(metaphylo, 0)
1×8 DataFrame
│ Row │ div_type            │ measure │ q     │ type_level │ type_name │ partition_level │ partition_name │ diversity │
│     │ String              │ String  │ Int64 │ String     │ String    │ String          │ String         │ Float64   │
├─────┼─────────────────────┼─────────┼───────┼────────────┼───────────┼─────────────────┼────────────────┼───────────┤
│ 1   │ Phylogenetic Branch │ Gamma   │ 0     │ types      │           │ metacommunity   │                │ 3.27211   │```

```@contents
```

```@autodocs
Modules = [Diversity]
Private = false
```

```@index
```
